// Copyright (C) 2018 Avatao.com Innovative Learning Kft.
// All Rights Reserved. See LICENSE file for details.

import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { MarkdownService } from '../services/markdown.service';
import { WebSocketService } from '../services/websocket.service';

import { MessagesMessage } from '../message-types/messages-message';
import { MessagesControlCommand } from '../message-types/messages-control-command';
import { config } from '../config';
import { CommandMessage } from '../message-types/command-message';

@Component({
  selector: 'app-messages',
  templateUrl: './messages.component.html',
  styleUrls: ['./messages.component.scss']
})
export class MessagesComponent implements OnInit {
  messages: MessagesMessage[] = [];
  showNextButton: boolean = config.messages.showNextButton;

  command_handlers = {
    'showNextButton': this.showButtonHandler.bind(this)
  };

  constructor(
    private markdownService: MarkdownService,
    private websocketService: WebSocketService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {
    this.websocketService.connect();
    this.websocketService.observeKey<MessagesMessage>('message').subscribe(
      (event) => this.handleMessage(event.data)
    );
    this.websocketService.observeKey<CommandMessage>('messagecontrol').subscribe(
      (event) => this.command_handlers[event.data.command](event.data)
    );
  }

  handleMessage(data: MessagesMessage) {
    data.message = this.convert(data.message);
    this.messages.push(data);
    this.changeDetectorRef.detectChanges();
  }

  convert(text: string) {
    return this.markdownService.convertToHtml(text);
  }

  showButtonHandler(data: MessagesControlCommand) {
    this.showNextButton = data.value;
  }

  stepFSM() {
    this.websocketService.sendJSON({key: '', trigger: 'step_next'});
  }
}
